/* AP 25-sep-12	istack.c			*/
/*		ein Integer-Stack		*/

# include <stdlib.h>
# include "istack.h"

enum errorT error;

void push( Stack s, int val ) {
	stackElemPtr pt;
	if (! s) {
		error = NOT_INITIALIZED;
		return;
	}
	pt = (stackElemPtr) malloc( sizeof( stackElemT ));
	if (! pt ) {
		error = NO_MEMORY;
		return;
	}
	pt->val = val;
	pt->prev = NULL;
	pt->next = s->top;
	if (s->top)
		s->top->prev = pt;
	s->top = pt;
	error = OK;
}
	
int pop( Stack s ) {
	int rc;
	stackElemPtr pt;
	if (! s ) {
		error = NOT_INITIALIZED;
		return 0;
	}
	if(! s->top) {
		error = EMPTY;
		return 0;
	}
	rc = s->top->val;
	pt = s->top;
	s->top = s->top->next;
	if (s->top)
		s->top->prev = NULL;
	free( pt );
	error = OK;
	return rc;
}
	
int top( Stack s ) {
	if (! s ) {
		error = NOT_INITIALIZED;
		return 0;
	}
	if(! s->top) {
		error = EMPTY;
		return 0;
	}
	error = OK;
	return s->top->val;
}

void swap( Stack s ) {
	stackElemPtr pt;
	if (! s ) {
		error = NOT_INITIALIZED;
		return;
	}
	if(! s->top) {
		error = EMPTY;
		return;
	}
	if (s && s->top && s->top->next) {
		pt = s->top;
		s->top = s->top->next;

		s->top->prev = pt->prev;
		pt->prev = s->top;
		pt->next = s->top->next;
		s->top->next = pt;
	}
	error = OK;
}
		
Stack allocStack() {
	Stack s = (Stack) malloc( sizeof( struct _stack ));
	if (! s) {
		error = NO_MEMORY;
		return (Stack) 0;
	}
	s->top = (stackElemPtr) 0;
	error = OK;
	return s;
}

void freeStack(Stack s) {
	stackElemPtr pt, p2;
	if (! s) {
		error = NOT_INITIALIZED;
		return;
	}
	pt = s->top;
	while (pt) {
		p2 = pt;
		pt = pt->next;
		free( p2 );
	}
	free( s );
	error = OK;
	return;
}
